import Taro, { Component, Config } from '@tarojs/taro';
import { ComponentClass } from 'react';
import { View, Text, Button, ScrollView, Image } from '@tarojs/components';
import { getPayOrder } from '../../api/Order/pay';
import { connect } from '@tarojs/redux';
import {
  Device,
  DeviceRefundMode,
  OnlyBluetooth,
} from '../../types/Device/Device';
import PrepayConfig from '../../types/Order/Order';
import { Customer } from '../../types/Customer/Customer';
import AES from 'crypto-js/aes';
import Utf8 from 'crypto-js/enc-utf8';
import ECBmode from 'crypto-js/mode-ecb';
import PaddingPkcs7 from 'crypto-js/pad-pkcs7';
// import { paySuccess } from '../../api/Order/paySuccess';
import './connect.scss';
import { str2ab, ab2str } from '@/utils/arrayBuffer';
import { DRY_SOCKET_URL } from '@/constants';
import selectIcon from '../../images/select_icon1@2x.png';
import selectedIcon from '../../images/selected_icon1@2x.png';
import select2Icon from '../../images/select_icon2@2x.png';
import selected2Icon from '../../images/selected_icon2@2x.png';

enum ContorlCode {
  using = '1',
  broken = '2',
  usable = '3',
  opened = '4',
}

type UploadMessage = {
  deviceCode: string;
  userId: string;
  type: number;
  contorlCode?: ContorlCode;
  data: string;
};

enum MessageType {
  info = 'info',
  success = 'success',
  error = 'error',
}
type Message = {
  type: MessageType;
  text: string;
};

enum BlueToothError {
  BlueToothNotOpen = 'BlueToothNotOpen',
  DeviceNotFound = 'DeviceNotFound',
}

type DeviceInfo = {
  deviceId: string;
  uuid: string;
  isPrimary: boolean;
  serviceId: string;
  notifyId: string;
  writeId: string;
  readId: string;
};
type PageStateProps = {
  device: Device;
  prepayConfig: PrepayConfig[];
  addprepayConfig: PrepayConfig[];
  userinfo: Customer;
};

type PageDispatchProps = {};

type PageState = {
  payId: undefined | number;
  payIds: number[];
  payMoney: number;
  bluetoothState: boolean;
  deviceInfo: DeviceInfo;
  sockedDone: boolean;
  deviceDone: boolean;
  showerState: boolean;
  messageQueue: Message[];
  lastMessage: string;
  canPay: boolean;
};

type PageOwnProps = {};

type Iprop = PageStateProps & PageDispatchProps & PageOwnProps;
interface Connect {
  props: Iprop;
  state: PageState;
}

const StopCode = 1000;
let timer: NodeJS.Timeout | null = null;
let reConnectting: boolean = false;
let socketTask: Taro.SocketTask | null = null;
let liveTimer: NodeJS.Timeout | null = null;
let msgList: Message[] = [];
let msgindex: number = 0;
let currentThis: Connect | null = null;

@connect(({ device, prepayConfig, userinfo }) => ({
  device,
  // prepayConfig: prepayConfig,
  prepayConfig: prepayConfig.filter((item: PrepayConfig) => item.isMajor === 1),
  addprepayConfig: prepayConfig.filter(
    (item: PrepayConfig) => item.isMajor === 2
  ),
  userinfo,
}))
class Connect extends Component {
  config: Config = {
    navigationBarTitleText: '设备链接',
  };
  constructor(props) {
    super(props);
    this.state = {
      payId: undefined,
      payIds: [],
      payMoney: 0,
      bluetoothState: false,
      sockedDone: socketTask ? true : false,
      deviceDone: false,
      showerState: false,
      deviceInfo: {
        deviceId: '',
        uuid: '',
        isPrimary: false,
        serviceId: '',
        notifyId: '',
        writeId: '',
        readId: '',
      },
      messageQueue: [],
      lastMessage: '',
      canPay: false,
    };
  }

  componentWillMount() {
    const { prepayConfig } = this.props;

    if (prepayConfig.length) {
      this.setState(
        {
          payId: prepayConfig[0].id,
        },
        () => {
          this.getPayMoney();
        }
      );
    }
    this.checkBluetoothAndWs();
    currentThis = this;
  }

  componentWillUnmount() {
    // this.closeBluetooth();
    // this.closeDeviceSocket();
    msgList = [];
    msgindex = 0;
    currentThis = null;
  }

  connectDeviceSocket() {
    const { device } = this.props;
    const { canPay } = this.state;
    // if (socketTask) return;
    // this.addMessageItem({
    //   type: MessageType.info,
    //   text: '服务器连接中...',
    // });
    if (msgindex) return;

    // Taro.closeSocket()
    //   .catch(err => {
    //     console.log(err);
    //   })
    //   .then(() => {

    //   })
    let connectHandle;
    if (socketTask) {
      connectHandle = new Promise(resolve => resolve(socketTask));
    } else {
      connectHandle = Taro.connectSocket({
        url: DRY_SOCKET_URL,
        timeout: 1200000,
      }).then(task => {
        socketTask = task;
        task.onOpen(() => {
          this.addMessageItem({
            type: MessageType.success,
            text: '服务器连接成功',
          });
          this.addConnectMsg();
          liveTimer = setInterval(() => {
            task.send({ data: str2ab('{}') });
          }, 20000);
          console.log('onOpen');
          if (reConnectting) {
            this.reConnectDeviceSocket(true);
          }
          this.setState({
            sockedDone: true,
          });
          task.send({ data: str2ab('{}') });
          this.sendDeviceCode();
        });
        task.onMessage(res => {
          const msg: string = ab2str(res.data);
          console.log('获取socket消息: ', msg);
          console.log('状态: ', canPay, device.isOnlyBluetooth);
          const msgObj: UploadMessage = JSON.parse(msg);
          if (msgObj.type === 1) {
            if (msgObj.data.length > 20) {
              for (
                let index = 0;
                index <= Math.floor(msgObj.data.length / 20);
                index++
              ) {
                let str = msgObj.data.substring(index * 20, (index + 1) * 20);
                this.sendMessageToDevice(str);
              }
            } else {
              this.sendMessageToDevice(msgObj.data);
            }
          } else if (msgObj.type === 0) {
            if (msgObj.contorlCode === ContorlCode.using) {
              Taro.showModal({
                title: '提示',
                content: '设备正在使用',
              });
            } else if (msgObj.contorlCode === ContorlCode.broken) {
              Taro.showModal({
                title: '提示',
                content: '设备已损坏',
              });
            } else if (msgObj.contorlCode === ContorlCode.usable) {
              console.log('in setState', currentThis.canPay);
              currentThis && currentThis.setState({ canPay: true });
            } else if (msgObj.contorlCode === ContorlCode.opened) {
              Taro.showModal({
                title: '提示',
                content: '设备已启动',
              });
            } else {
            }
          } else {
          }
        });
        task.onClose(e => {
          if (msgindex !== 0) {
            this.addMessageItem({
              type: MessageType.error,
              text: '服务器连接失败，请稍后扫码重试',
            });
          }
          console.log('socked关闭', e, reConnectting, timer);
          this.setState({
            sockedDone: false,
          });
          liveTimer && clearInterval(liveTimer);
          socketTask = null;
          liveTimer = null;
          if (e.code === StopCode) {
            console.log('正确结束socket连接');
          } else {
            console.log('开始重连socket');
            this.reConnectDeviceSocket(false);
          }
        });
      });
    }
    connectHandle.then(() => {
      console.log('socket done');
    });
  }

  addConnectMsg() {
    const { sockedDone, deviceDone } = this.state;
    if (sockedDone && deviceDone) {
      if (msgindex === 0) {
        this.addMessageItem({
          type: MessageType.info,
          text: '正在与设备通信...',
        });
      }
      msgindex++;
    }
  }

  reConnectDeviceSocket(cancel: boolean) {
    if (cancel) {
      timer && clearTimeout(timer);
      timer = null;
      reConnectting = false;
      return;
    }
    console.log(reConnectting, timer);

    if (reConnectting) {
      this.connectDeviceSocket();
    } else if (timer) {
      clearTimeout(timer);
      reConnectting = false;
      timer = null;
      console.log('请保证网络正常');
      Taro.showModal({
        title: '警告',
        content: '请保持网络畅通正常',
      });
    } else {
      timer = setTimeout(() => {
        reConnectting = false;
      }, 10000);
      reConnectting = true;
      console.log(reConnectting, timer);
      this.connectDeviceSocket();
    }
  }

  sendDeviceCode() {
    const { sockedDone, deviceDone } = this.state;
    // const { device } = this.props;
    console.log('socket状态：', sockedDone, '蓝牙状态：', deviceDone);

    // if (sockedDone && socketTask && deviceDone) {
    //   let deviceData = '{<' + device.code + '>}';
    //   console.log('<---发送设备编号:', deviceData);
    //   socketTask.send({
    //     data: str2ab(deviceData),
    //     success: msg => {
    //       console.log('发送设备编号:', msg, '--->');
    //     },
    //     fail: err => {
    //       console.log('发送设备编号:', err, '--->');
    //     },
    //   });
    // }
  }

  closeDeviceSocket() {
    console.log('in close', socketTask, timer);
    if (socketTask) {
      socketTask.close({
        code: StopCode,
        complete: () => {
          socketTask = null;
        },
      });
    }
    if (timer) {
      clearTimeout(timer);
    }
  }
  checkBluetoothAndWs() {
    this.closeBluetooth()
      .then(() => {
        console.log('打开蓝牙和WebSocket');
        this.openBluetooth();
        this.connectDeviceSocket();
      })
      .catch(err => {
        console.log(err);
        console.log(err, '打开蓝牙和WebSocket');
        this.openBluetooth();
        this.connectDeviceSocket();
      });
  }

  openBluetooth() {
    this.addMessageItem({
      type: MessageType.info,
      text: '蓝牙链接中...',
    });
    Taro.openBluetoothAdapter()
      .then(res => {
        console.log(res);
        this.setState(
          {
            bluetoothState: true,
          },
          () => {
            this.startConnectDevice();
          }
        );
      })
      .catch(err => {
        this.addMessageItem({
          type: MessageType.error,
          text: '蓝牙连接失败，请稍后扫码重试',
        });
        Taro.showModal({
          title: '温馨提示',
          content: '为了更好的体验本次服务,请打开手机蓝牙!',
        });
        console.error(err);
        Taro.onBluetoothAdapterStateChange(res => {
          console.log('adapterState changed, now is', res);
          if (res.available) {
            this.setState({
              bluetoothState: true,
            });
          }
        });
      });
  }

  checkBluetoothState() {
    const { bluetoothState } = this.state;
    return bluetoothState;
  }

  closeBluetoothConnection() {
    const { deviceInfo } = this.state;
    this.setState({
      showerState: false,
    });
    Taro.closeBLEConnection({
      deviceId: deviceInfo.deviceId,
    })
      .then(() => {
        this.setState({
          deviceDone: false,
        });
      })
      .catch(err => {
        console.log('断开蓝牙连接失败！');
        console.error(err);
      });
  }

  closeBluetooth() {
    return Taro.closeBluetoothAdapter();
  }

  warnBluetoothOpen() {
    return Taro.showModal({
      title: '警告',
      content: '为保障正常使用，请打开蓝牙功能',
    });
  }

  startDevicesDiscovery() {
    console.log('in startDevicesDiscovery', this.checkBluetoothState());

    if (this.checkBluetoothState()) {
      return Taro.startBluetoothDevicesDiscovery({
        // services: ['00001103-0000-1000-8000-00805f9b34fb'],
      })
        .then(() => {
          const {
            code,
            // isOnlyBluetooth
          } = this.props.device;
          let timer: NodeJS.Timeout | null = null;
          return new Promise((resolve, reject) => {
            // if (isOnlyBluetooth) {
            //   timer = setTimeout(() => {
            //     reject({ msg: BlueToothError.DeviceNotFound });
            //   }, 10000);
            // } else {
            timer = setTimeout(() => {
              reject({ msg: BlueToothError.DeviceNotFound });
            }, 5000);
            // }
            Taro.onBluetoothDeviceFound(res => {
              const devices = res.devices;
              if (devices[0].name === code) {
                let deviceId = devices[0].deviceId;
                Taro.stopBluetoothDevicesDiscovery();
                this.setState({
                  deviceId,
                });
                if (timer) {
                  clearInterval(timer);
                }
                resolve(deviceId);
              }
            });
          });
        })
        .catch(err => {
          // Taro.stopBluetoothDevicesDiscovery();
          if (timer) {
            clearInterval(timer);
          }
          Taro.hideLoading();
          console.error('startBluetoothDevicesDiscovery: ', err);
          // if (err.msg === BlueToothError.DeviceNotFound) {
          Taro.stopBluetoothDevicesDiscovery();
          // this.sendStartShower();
          throw err;
          // } else {
          //   return this.warnBluetoothOpen();
          // }
        });
    } else {
      Taro.hideLoading();
      return this.warnBluetoothOpen().then(res => {
        if (res.confirm) {
          // const { isOnlyBluetooth } = this.props.bluetoothDevice;
          // if (!isOnlyBluetooth) {
          //   Taro.showLoading({
          //     title: '开启中',
          //     mask: true,
          //   });
          //   return new Promise((resolve, reject) => {
          //     setTimeout(() => {
          //       this.sendStartShower();
          //       reject();
          //     }, 5000);
          //   });
          // }
        }
        throw BlueToothError.BlueToothNotOpen;
      });
    }
  }

  createConnection(deviceId: string) {
    const { userinfo, device } = this.props;
    console.log(deviceId);
    return new Promise((resolve, reject) => {
      Taro.createBLEConnection({
        deviceId,
      });

      let abStr = '';
      Taro.onBLECharacteristicValueChange(res => {
        console.log(
          `characteristic ${res.characteristicId} has changed, now is ${res.value}`
        );
        console.log(res.value);
        let datastr = ab2str(res.value);
        console.log('获取bluetooth消息： ', datastr);
        if (datastr === '<>') {
          console.log('结束蓝牙以及socket: ', datastr);
          // this.closeDeviceSocket();
          this.closeBluetoothConnection();
        } else {
          if (
            (datastr.substring(0, 1) === '{' &&
              datastr.substring(datastr.length - 1, datastr.length) === '}') ||
            (datastr.substring(0, 1) === '<' &&
              datastr.substring(datastr.length - 1, datastr.length) === '>')
          ) {
            console.log('<---发送完整数据： ', datastr);
            if (socketTask) {
              let sendData: UploadMessage = {
                deviceCode: device.code,
                userId: userinfo.customerId.toString(),
                type: 1,
                data: datastr,
              };
              socketTask.send({
                data: str2ab(JSON.stringify(sendData)),
                success: msg => {
                  console.log('发送完整数据：', msg, '--->');
                },
                fail: err => {
                  console.log('发送完整数据：', err, '--->');
                },
              });
            }
          } else if (
            datastr.substring(0, 1) === '{' ||
            datastr.substring(0, 1) === '<'
          ) {
            abStr = datastr;
          } else if (
            (abStr.length &&
              datastr.substring(datastr.length - 1, datastr.length) === '}') ||
            (abStr.length &&
              datastr.substring(datastr.length - 1, datastr.length) === '>')
          ) {
            abStr += datastr;
            console.log('<---发送完整数据： ', abStr);
            let sendData: UploadMessage = {
              deviceCode: device.code,
              userId: userinfo.customerId.toString(),
              type: 1,
              data: abStr,
            };
            if (socketTask) {
              socketTask.send({
                data: str2ab(JSON.stringify(sendData)),
                success: msg => {
                  console.log('发送完整数据：', msg, '--->');
                },
                fail: err => {
                  console.log('发送完整数据：', err, '--->');
                },
              });
            }
            abStr = '';
          } else if (abStr.length) {
            console.log('追加数据：', datastr);
            abStr += datastr;
          }
        }
      });

      Taro.onBLEConnectionStateChange(res => {
        // const { showerState } = this.state;
        console.log(
          `device ${res.deviceId} state has changed, connected: ${res.connected}`
        );
        if (res.connected) {
          this.setState({
            deviceDone: true,
            showerState: true,
          });
          resolve(res.deviceId);
        } else {
          console.log(res);
          this.setState({
            deviceDone: false,
          });
          // if (showerState) {
          this.reConnectDeviceBluetooth(); // 只要断了就重连
          // }
          reject('蓝牙未成功连接');
        }
      });
    });
  }

  sendMessageToDevice(msg: string) {
    const { deviceInfo } = this.state;
    console.log(msg, str2ab(msg).byteLength);
    if (deviceInfo.writeId) {
      Taro.writeBLECharacteristicValue({
        deviceId: deviceInfo.deviceId,
        serviceId: deviceInfo.serviceId,
        characteristicId: deviceInfo.writeId,
        value: str2ab(msg),
      });
    }
  }

  reConnectDeviceBluetooth() {
    if (this.checkBluetoothState()) {
      const { deviceInfo } = this.state;
      this.createConnection(deviceInfo.deviceId);
    }
  }

  getDeviceServices(deviceId: string) {
    return Taro.getBLEDeviceServices({
      deviceId: deviceId,
    }).then(services => {
      return { deviceId: deviceId, services: services.services };
    });
  }

  getDeviceCharacter({
    deviceId,
    serviceId,
  }: {
    deviceId: string;
    serviceId: string;
  }) {
    console.log(deviceId, serviceId);
    return Taro.getBLEDeviceCharacteristics({
      deviceId,
      serviceId,
    }).then(res => {
      let notifyId = '',
        writeId = '',
        readId = '';
      for (let i = 0; i < res.characteristics.length; i++) {
        let charc = res.characteristics[i];
        if (charc.properties.notify) {
          notifyId = charc.uuid;
        }
        if (charc.properties.write) {
          writeId = charc.uuid;
        }
        if (charc.properties.read) {
          readId = charc.uuid;
        }
      }
      return {
        notifyId,
        writeId,
        readId,
      };
    });
  }

  openNotify({ notifyId, writeId, readId }) {
    console.log(notifyId, writeId, readId);
    this.setState(({ deviceInfo }: PageState) => ({
      deviceInfo: {
        ...deviceInfo,
        notifyId,
        writeId,
        readId,
      },
    }));
    const { deviceInfo } = this.state;
    return Taro.notifyBLECharacteristicValueChange({
      deviceId: deviceInfo.deviceId,
      serviceId: deviceInfo.serviceId,
      characteristicId: notifyId,
      state: true,
    });
  }

  userWarnningHandle(thresholdPrompt, money, beanAmount) {
    Taro.hideLoading();
    console.log('in userWarnningHandle');
    let arr = [money, beanAmount];
    let i = -1;
    return Taro.showModal({
      title: '提示',
      content: thresholdPrompt.replace(/(\{.*?\})/g, () => {
        i++;
        return arr[i];
      }),
    }).then(res => {
      if (res.confirm) {
        Taro.showLoading({
          title: '开启中',
          mask: true,
        });
        return Promise.resolve();
      } else {
        return Promise.reject();
      }
    });
  }

  startConnectDevice() {
    const { sockedDone } = this.state;
    if (!sockedDone) {
      this.connectDeviceSocket();
    }
    this.startDevicesDiscovery()
      .then((deviceId: string) => this.createConnection(deviceId))
      .then(this.getDeviceServices)
      .then(res => {
        const deviceId = res.deviceId;
        const services = res.services;
        let service = services.find(
          item =>
            item.uuid.toLocaleLowerCase() ===
            '6e401103-b5a3-f393-e0a9-e50e24dcca9e'
        );
        if (!service) {
          service = services[0];
        }
        console.log('获取全部services: ', services, '. serviceId: ', service);
        const serviceId = service.uuid;
        this.setState(({ deviceInfo }: PageState) => ({
          deviceInfo: {
            ...deviceInfo,
            deviceId,
            serviceId,
          },
        }));
        return this.getDeviceCharacter({
          deviceId,
          serviceId,
        });
      })
      .then(res => this.openNotify(res))
      .then(() => {
        this.addMessageItem({
          type: MessageType.success,
          text: '蓝牙连接成功',
        });
        this.addConnectMsg();
        this.sendDeviceCode();
        // if (!reConnect) {
        //   setTimeout(() => {
        //     this.sendStartShower();
        //   }, 2000);
        // }
      })
      .catch(err => {
        console.error(err);
        Taro.hideLoading();
      });
  }

  sendStartShower() {
    console.log('开启洗浴');
    // const { code } = this.props.device;
    // const { userinfo } = this.props;
    // startShowerEquipment({
    //   deviceCode: code,
    //   customerId: userinfo.customerId.toString(),
    //   customerName: userinfo.customerName,
    //   customerPhone: userinfo.customerPhone,
    // })
    //   .then(res => {
    //     console.log(res);
    //     Taro.showToast({
    //       title: '开启成功！',
    //     });
    //     this.setState({
    //       showerState: true,
    //     });
    //   })
    //   .catch(err => {
    //     console.error(err);
    //     Taro.showToast({
    //       title: err.msg || '开启失败！',
    //       icon: 'none',
    //     });
    //     this.closeBluetoothConnection();
    //   });
  }

  selectPayConfig(id: number) {
    if (id === this.state.payId) return;
    this.setState(
      {
        payId: id,
      },
      () => {
        this.getPayMoney();
      }
    );
  }

  getPayMoney() {
    let money = 0;
    if (this.state.payId) {
      let payConfig = this.props.prepayConfig.find(
        payConfig => payConfig.id === this.state.payId
      );
      money += payConfig ? payConfig.prepayMoney : 0;
    }
    if (this.state.payIds.length) {
      this.props.addprepayConfig
        .filter(payConfig => this.state.payIds.includes(payConfig.id))
        .map(payConfig => {
          money += payConfig.prepayMoney;
          return payConfig;
        });
    }

    this.setState({
      payMoney: money,
    });
  }

  checkAddPayConfig(id: number) {
    let index = this.state.payIds.findIndex(payId => payId === id);
    let payIds: number[] = [];
    if (index === -1) {
      payIds = [...this.state.payIds, id];
    } else {
      payIds = [
        ...this.state.payIds.slice(0, index),
        ...this.state.payIds.slice(index + 1),
      ];
    }
    this.setState(
      {
        payIds,
      },
      () => {
        this.getPayMoney();
      }
    );
  }

  showerPayHandle() {
    this.closeBluetooth();
    return Taro.showModal({
      title: '提示',
      content: '设备蓝牙或网络连接失败...请稍后再试',
      showCancel: false,
    }).then(res => {
      console.log(res);
      this.openBluetooth();
    });
  }

  payHandle(phoneData) {
    const {
      device,
      userinfo: { customerId },
    } = this.props;
    const { payId, payIds, deviceDone, sockedDone } = this.state;
    if (payId) {
      if (device.isOnlyBluetooth && device.isOnlyBluetooth === 1) {
        console.log(deviceDone, sockedDone);
        if (!deviceDone || !sockedDone) {
          return this.showerPayHandle();
        }
      }
      let entity = {
        equipmentNum: device.originCode,
        equipmentPos: device.position,
        payType: '2',
        serviceId: device.serviceId,
        smaproCustomerId: customerId,
        smaproPrepayConfigId: payId,
        smaproPrepayConfigIds: [...payIds, payId],
        areaId: device.areaId,
        areaName: device.areaName,
      };
      if (device.rates[0]) {
        entity.duration = device.rates[0].duration;
      }
      if (phoneData) {
        entity = { ...entity, ...phoneData };
      }
      getPayOrder(entity)
        .then(res => {
          console.log('res: ', res);

          const { payStr } = res;
          let key = customerId.toString();
          while (key.length < 16) {
            key = key + '0';
          }

          const payData = JSON.parse(
            AES.decrypt(payStr, Utf8.parse(key), {
              mode: ECBmode,
              padding: PaddingPkcs7,
            }).toString(Utf8)
          );
          console.log('payData:', payData);
          Taro.requestPayment({
            timeStamp: payData.msg.timeStamp.toString(),
            nonceStr: payData.msg.nonceStr,
            package: payData.msg.package,
            signType: payData.msg.signType,
            paySign: payData.msg.paySign,
          }).then(res => {
            console.log(res);
            // paySuccess({ orderId: outTradeNo });
            // Taro.showLoading({
            //   title: '设备连接中',
            //   mask: true,
            // });
            Taro.showToast({
              title: '支付成功！',
              icon: 'success',
              mask: true,
            });
            this.gobackHandle();
          });
        })
        .catch(err => {
          Taro.showToast({
            title: '发起支付失败!',
            icon: 'none',
          });
          console.error(err);
        });
    } else {
      Taro.showToast({
        title: '请选择预付金额',
        icon: 'none',
      });
    }
  }

  changePageState(state: boolean) {
    if (state) {
      Taro.setNavigationBarTitle({
        title: '设备使用',
      });
    } else {
      Taro.setNavigationBarTitle({
        title: '设备连接',
      });
    }
    this.setState({
      canPay: state,
    });
  }

  gobackHandle() {
    msgList = [];
    setTimeout(() => {
      this.setState({
        canPay: false,
      });
      Taro.redirectTo({
        url: '/pages/index/index?', // 需要问号不然回到首页可以取到业务参数
      });
    }, 2000);
  }

  getPhoneHandle(e) {
    console.log(e);
    const { encryptedData, iv } = e.detail;
    if (encryptedData && iv) {
      console.log('获得手机权限');
      this.payHandle({
        encryptedData,
        iv,
      });
    }
  }

  addMessageItem(msg: Message) {
    msgList.push(msg);
    this.setState({
      messageQueue: [...msgList],
      lastMessage: 'Connect-Message-item' + msgList.length,
    });
  }

  render() {
    const { device, prepayConfig, addprepayConfig, userinfo } = this.props;
    const {
      payId,
      payIds,
      messageQueue,
      lastMessage,
      canPay,
      payMoney,
    } = this.state;
    console.log(canPay, device.isOnlyBluetooth);

    return (
      <View className="Connect">
        <View className="state">{canPay}</View>
        {/* <Button onClick={() => currentThis.gobackHandle()}>重载</Button> */}
        {device.isOnlyBluetooth && canPay ? (
          <View>
            <View className="Connect-title">{`${
              device.serviceName ? device.serviceName : ''
            } ${device.code ? device.code : ''}(${device.position})`}</View>
            <View className="Connect-tip3">
              说明：请在1分钟内选择功能并进行支付；支付后30秒内设备没有正常启动，重新扫码后无需支付可自动开启设备。
            </View>
            <View className="Connect-Pay">
              <View className="Connect-Pay-title">
                请选择主功能
                <Text className="Connect-Pay-titleTip">(只能选择一个)</Text>
              </View>
              <View className="Connect-Pay-content">
                {prepayConfig &&
                  prepayConfig.length &&
                  prepayConfig.map(payConfig => (
                    <View
                      className="Connect-Pay-item"
                      key="id"
                      onClick={() => this.selectPayConfig(payConfig.id)}>
                      <View className="Connect-Pay-itemName">
                        {payConfig.name}
                      </View>
                      <View className="Connect-Pay-itemTime">
                        {payConfig.duration}分钟
                      </View>
                      <View className="Connect-Pay-itemPrice">
                        {payConfig.prepayMoney.toFixed(2)}元
                      </View>
                      <View className="Connect-Pay-itemSelect">
                        {payId === payConfig.id ? (
                          <Image
                            className="Connect-Pay-itemSelecticon"
                            src={selectedIcon}></Image>
                        ) : (
                          <Image
                            className="Connect-Pay-itemSelecticon"
                            src={selectIcon}></Image>
                        )}
                      </View>
                    </View>
                  ))}
              </View>
            </View>
            <View className="Connect-Pay">
              <View className="Connect-Pay-title">请选择附加功能</View>
              <View className="Connect-Pay-content">
                {addprepayConfig &&
                  addprepayConfig.length &&
                  addprepayConfig.map(payConfig => (
                    <View
                      className="Connect-Pay-item"
                      key="id"
                      onClick={() => this.checkAddPayConfig(payConfig.id)}>
                      <View className="Connect-Pay-itemName">
                        {payConfig.name}
                      </View>
                      <View className="Connect-Pay-itemPrice">
                        {payConfig.prepayMoney.toFixed(2)}元
                      </View>
                      <View className="Connect-Pay-itemSelect">
                        {payIds.find(id => id === payConfig.id) ? (
                          <Image
                            className="Connect-Pay-itemSelecticon"
                            src={selected2Icon}></Image>
                        ) : (
                          <Image
                            className="Connect-Pay-itemSelecticon"
                            src={select2Icon}></Image>
                        )}
                      </View>
                    </View>
                  ))}
              </View>
            </View>
            {userinfo.isAuth ? (
              <Button className="Connect-btn" onClick={this.payHandle}>
                确定支付 共{payMoney.toFixed(2)}元
              </Button>
            ) : (
              <Button
                className="Connect-btn"
                open-type="getPhoneNumber"
                onGetphonenumber={this.getPhoneHandle}>
                确定支付 共{payMoney.toFixed(2)}元
              </Button>
            )}
          </View>
        ) : (
          <View>
            <View className="Connect-info">
              <View className="Connect-info-item">
                <Text>服务类型</Text>
                <Text>{device.serviceName ? device.serviceName : ''}</Text>
              </View>
              <View className="Connect-info-item">
                <Text>设备编号</Text>
                <Text>{device.code ? device.code : ''}</Text>
              </View>
              <View className="Connect-info-item">
                <Text>设备位置</Text>
                <Text className="Connect-info-position">{device.position}</Text>
              </View>
            </View>
            <ScrollView
              scroll-y
              scroll-into-view={lastMessage}
              className="Connect-Message">
              {messageQueue.map((item, index) => (
                <View
                  key={'Connect-Message-item' + index}
                  id={'Connect-Message-item' + index}
                  className={'Connect-Message-item ' + item.type}>
                  {item.text}
                </View>
              ))}
            </ScrollView>
            <View className="Connect-tip2">
              说明：如果在1分钟内没有跳转到支付页面，请开启手机蓝牙后再次微信扫码。
            </View>
          </View>
        )}
      </View>
    );
  }
}

export default Connect as ComponentClass<Iprop, PageState>;
